% relaxation.m
% illustrate the behavior of
% the "classical" iterative methods
%    Jacobi, Gauss Seidel, relaxation
% the matrix A initialized by PDE Toolbox
% (Dirichlet b.v.p. in a rectangular domain)
% pde initial data stored in 'pde_init.mat'
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all; close all;
load('pde_init');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% refine the grid
[p,e,t] = refinemesh(g,p,e,t);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[A,F,B,ud] = assempde(b,p,e,t,c,a,f);

u1 = A\F     ; % find the non-dirichlet points
solution = B*u1 + ud; % full solution

n = length(u1); % dimension

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% now go by iteration
% decompose the matrix A
L = tril(A,-1);
R = triu(A,1);
D = tril(A,0)-L;

maxit = 20; 

omega = 1.5; % relaxation parameter

res.Jacobi = zeros(maxit+1,1);
res.GS     = zeros(maxit+1,1);
res.relax  = zeros(maxit+1,1);

u.Jacobi = zeros(n,1); res.Jacobi(1) = norm(A*u.Jacobi - F);
u.GS     = zeros(n,1); res.GS(1)     = norm(A*u.GS - F);
u.relax  = zeros(n,1); res.relax(1)  = norm(A*u.relax - F);

for ctr = 1:maxit
    u.Jacobi = D\(F - (L+R)*u.Jacobi);
    res.Jacobi(ctr+1) = norm( A*u.Jacobi - F);
    
    u.GS = (D+L)\(F- R*u.GS);
    res.GS(ctr+1) = norm( A*u.GS - F);
    
    u.relax = (D+omega*L)\( omega*F + ((1-omega)*D - omega*R)*u.relax );
    res.relax(ctr+1) = norm( A*u.relax - F);
    
    sol.Jacobi = B*u.Jacobi;
    sol.GS = B*u.GS;
    sol.relax = B*u.relax;
    
    if 0
        figure(1)
        subplot(1,3,1)
        pdesurf(p,t,solution-sol.Jacobi);
        shading faceted;
        axis([-0.9, 0.5 -0.4 0.5 0 0.5]);
        title('Jacobi')

        subplot(1,3,2)
        pdesurf(p,t,solution-sol.GS);
        shading faceted;
        axis([-0.9, 0.5 -0.4 0.5 0 0.5]);
        title('Gauss Seidel')

        subplot(1,3,3);
        pdesurf(p,t,solution-sol.relax);
        shading faceted;
        axis([-0.9, 0.5 -0.4 0.5 0 0.5]);
        title('relaxation')

        colormap jet;
        drawnow;
        pause(0.1);
        %waitforbuttonpress
    end
end


figure(2);
I = (1:maxit+1)';

semilogy(I,res.Jacobi,'b',I,res.GS,'g',I,res.relax,'r');
legend('Jacobi','GS','relax');


